package me.mrletsplay.nojs;

import java.io.IOException;
import java.io.InputStream;

import org.slf4j.helpers.NOPLogger;

import me.mrletsplay.nojs._test.DynPage;
import me.mrletsplay.nojs._test.LinksPage;
import me.mrletsplay.nojs._test.TestPage;
import me.mrletsplay.nojs.action.Action;
import me.mrletsplay.nojs.action.ActionData;
import me.mrletsplay.nojs.component.impl.Button;
import me.mrletsplay.nojs.component.impl.Button.Style;
import me.mrletsplay.nojs.component.impl.Form;
import me.mrletsplay.nojs.component.impl.Link;
import me.mrletsplay.nojs.component.impl.TextInput;
import me.mrletsplay.nojs.page.StaticPage;
import me.mrletsplay.simplehttpserver.dom.html.HtmlDocument;
import me.mrletsplay.simplehttpserver.dom.html.HtmlElement;
import me.mrletsplay.simplehttpserver.dom.html.element.HtmlButton;
import me.mrletsplay.simplehttpserver.http.HttpRequestMethod;
import me.mrletsplay.simplehttpserver.http.HttpStatusCodes;
import me.mrletsplay.simplehttpserver.http.header.DefaultClientContentTypes;
import me.mrletsplay.simplehttpserver.http.request.HttpRequestContext;
import me.mrletsplay.simplehttpserver.http.request.urlencoded.UrlEncoded;
import me.mrletsplay.simplehttpserver.http.response.HtmlResponse;
import me.mrletsplay.simplehttpserver.http.response.TextResponse;
import me.mrletsplay.simplehttpserver.http.server.HttpServer;
import me.mrletsplay.simplehttpserver.http.util.MimeType;

public class NoJS {

	public static void main(String[] args) {
		HttpServer httpServer = new HttpServer(HttpServer.newConfigurationBuilder()
			.hostBindAll()
			.port(6969)
			.poolSize(10)
			.ioWorkers(2)
			.debugMode(true)
			.logger(NOPLogger.NOP_LOGGER)
			.create());

		httpServer.getDocumentProvider().register(HttpRequestMethod.GET, "/", () -> {
			HttpRequestContext ctx = HttpRequestContext.getCurrentContext();
			HtmlDocument doc = new HtmlDocument();

			doc.setTitle("Test");
			doc.setIcon("/amogus.png");
			doc.setDescription("Just testing");

			HtmlElement form = HtmlElement.of("form");
			form.setAttribute("method", "post");

			HtmlElement input = HtmlElement.of("input");
			input.setAttribute("name", "texttest");
			form.appendChild(input);

			HtmlButton button = HtmlElement.button();
			button.setText("Click me!");
			form.appendChild(button);

			doc.getBodyNode().appendChild(form);

			ctx.respond(HttpStatusCodes.OK_200, new HtmlResponse(doc));
		});

//		new TestEndpoint().register(httpServer.getDocumentProvider());

		StaticPage test = new StaticPage()
			.add(new Button().text("Click me!").action(new Action("poopy").extraData(new ActionData().put("test", "abc"))).style(Style.LINK))
			.add(new Link().text("Click me!").href("#"))
			.add(new Form()
				.add(new TextInput("username").placeholder("Username"))
				.add(new TextInput("password").placeholder("Password"))
				.add(new Button().text("Login"))
				.add(new Button().text("Can't click here").enabled(false))
				.action(new Action("beans"))
			);
		test.register(httpServer.getDocumentProvider(), "/test");

		new TestPage().register(httpServer.getDocumentProvider(), "/test2");
		new DynPage().register(httpServer.getDocumentProvider(), "/dyn");
		new LinksPage().register(httpServer.getDocumentProvider(), "/links");

		httpServer.getDocumentProvider().register(HttpRequestMethod.POST, "/", () -> {
			HttpRequestContext ctx = HttpRequestContext.getCurrentContext();
			UrlEncoded data = ctx.getClientHeader().getPostData().getParsedAs(DefaultClientContentTypes.URLENCODED);
			ctx.respond(HttpStatusCodes.OK_200, new TextResponse(data.toString()));
		});

		httpServer.getDocumentProvider().register(HttpRequestMethod.GET, "/base.css", () -> {
			try(InputStream in = NoJS.class.getResourceAsStream("/style/base.css")) {
				HttpRequestContext ctx = HttpRequestContext.getCurrentContext();
				ctx.getServerHeader().setContent(MimeType.CSS, in.readAllBytes());
			} catch(IOException ex) {
				throw new RuntimeException(ex);
			}
		});

//		httpServer.getDocumentProvider().registerPattern(HttpRequestMethod.GET, "/{path...?}", () -> {
//			HttpRequestContext ctx = HttpRequestContext.getCurrentContext();
//			ctx.respond(HttpStatusCodes.OK_200, new TextResponse(ctx.getPathParameters().get("path")));
//		});

		httpServer.start();
	}

}
