package me.mrletsplay.nojs._test;

import java.util.ArrayList;
import java.util.List;

import me.mrletsplay.nojs.action.Action;
import me.mrletsplay.nojs.action.ActionEvent;
import me.mrletsplay.nojs.action.ActionHandler;
import me.mrletsplay.nojs.component.Component;
import me.mrletsplay.nojs.component.impl.Button;
import me.mrletsplay.nojs.component.impl.Form;
import me.mrletsplay.nojs.component.impl.Group;
import me.mrletsplay.nojs.component.impl.Link;
import me.mrletsplay.nojs.component.impl.Message;
import me.mrletsplay.nojs.component.impl.Text;
import me.mrletsplay.nojs.component.impl.TextInput;
import me.mrletsplay.nojs.page.Page;
import me.mrletsplay.nojs.page.layout.GridLayout;
import me.mrletsplay.nojs.page.layout.Layout;

public class LinksPage implements Page {

	private List<String> links = new ArrayList<>();
	private String error;

	@Override
	public List<Component> getComponents() {
		Group linksGroup = new Group()
			.layout(GridLayout.ofColumns(1)
				.gap("2px"));

		for(String s : links) {
			linksGroup.add(new Link().href(s).text(s));
		}

		if(links.isEmpty()) {
			linksGroup.add(new Text().text("No links added yet"));
		}

		if(error != null) {
			linksGroup.add(new Message().text(error));
			error = null;
		}

		return List.of(
			linksGroup,
			new Form()
				.add(new Text().text("Add a new link:"))
				.add(new Group()
					.add(new TextInput("link").placeholder("Insert link here"))
					.add(new Button().text("Add Link"))
					.layout(new GridLayout().columns("1fr", "auto").gap("4px"))
				)
				.layout(GridLayout.ofColumns(1).gap("4px"))
				.action(new Action("addLink"))
		);
	}

	@Override
	public Layout getLayout() {
		return GridLayout.ofColumns(1)
			.gap("10px");
	}

	@ActionHandler("addLink")
	public void addLink(ActionEvent event) {
		String link = event.getData("link");
		if(link == null || link.isBlank()) {
			error = "Invalid link";
			return;
		}

		links.add(event.getData("link"));
	}

}
