package me.mrletsplay.nojs.page;

import java.util.List;

import me.mrletsplay.nojs.action.ActionEvent;
import me.mrletsplay.nojs.action.ActionHandlers;
import me.mrletsplay.nojs.component.Component;
import me.mrletsplay.nojs.page.layout.Layout;
import me.mrletsplay.simplehttpserver.dom.css.CssElement;
import me.mrletsplay.simplehttpserver.dom.css.CssSelector;
import me.mrletsplay.simplehttpserver.dom.css.StyleSheet;
import me.mrletsplay.simplehttpserver.dom.html.HtmlDocument;
import me.mrletsplay.simplehttpserver.dom.html.HtmlElement;
import me.mrletsplay.simplehttpserver.http.HttpRequestMethod;
import me.mrletsplay.simplehttpserver.http.HttpStatusCodes;
import me.mrletsplay.simplehttpserver.http.document.DocumentProvider;
import me.mrletsplay.simplehttpserver.http.document.HttpDocument;
import me.mrletsplay.simplehttpserver.http.header.DefaultClientContentTypes;
import me.mrletsplay.simplehttpserver.http.request.HttpRequestContext;
import me.mrletsplay.simplehttpserver.http.request.urlencoded.UrlEncoded;
import me.mrletsplay.simplehttpserver.http.response.HtmlResponse;

public interface Page extends HttpDocument, ActionHandlers {

	public List<Component> getComponents();

	public default Layout getLayout() {
		return null;
	}

	public default HtmlDocument toHtml() {
		HtmlDocument doc = new HtmlDocument();
		doc.addStyleSheet("/base.css");

		StyleSheet sheet = new StyleSheet();

		CssElement cssE = new CssElement(CssSelector.selectClass("ee"));
		cssE.setProperty("text", "red");
		sheet.addElement(cssE);

		doc.getHeadNode().appendChild(HtmlElement.style(sheet));

		Layout layout = getLayout();
		if(layout != null) {
			layout.apply(doc.getBodyNode());
		}

		for(Component c : getComponents()) {
			doc.getBodyNode().appendChild(c.toHtml());
		}

		return doc;
	}

	@Override
	public default void createContent() {
		HttpRequestContext ctx = HttpRequestContext.getCurrentContext();

		if(ctx.getClientHeader().getMethod() == HttpRequestMethod.POST) {
			// TODO: error handling
			UrlEncoded data = ctx.getClientHeader().getPostData().getParsedAs(DefaultClientContentTypes.URLENCODED);
			System.out.println(data);

			ActionEvent e = new ActionEvent(data);
			onAction(e);

			ctx.redirect(HttpStatusCodes.SEE_OTHER_303, ctx.getClientHeader().getPath().toString());
			return;
		}

		ctx.respond(HttpStatusCodes.OK_200, new HtmlResponse(toHtml()));
	}

	public default void register(DocumentProvider provider, String path) {
		provider.register(HttpRequestMethod.GET, path, this);
		provider.register(HttpRequestMethod.POST, path, this);
	}

}
